@@ -38,7 +38,7 @@ from requests.adapters import HTTPAdapter
 from requests.packages.urllib3.util.retry import Retry
 
 
-VERSION = '2.0.0'
+VERSION = '2.0.1'
 MANIFEST_LOCKH = None
 REPO_LOCKH = dict()
 GITBIN = '/usr/bin/git'
@@ -558,18 +558,29 @@ def find_siblings(fullpath, my_roots, known_roots, exact=False):
     siblings = set()
     for gitpath, gitroots in known_roots.items():
         # Of course we're going to match ourselves
-        if fullpath == gitpath or not my_roots or not gitroots:
+        if fullpath == gitpath or not my_roots or not gitroots or not len(gitroots.intersection(my_roots)):
             continue
-        if exact and gitroots == my_roots:
+        if gitroots == my_roots:
             siblings.add(gitpath)
-        elif not exact and len(gitroots.intersection(my_roots)):
+            continue
+        if exact:
+            continue
+        if gitroots.issubset(my_roots) or my_roots.issubset(gitroots):
             siblings.add(gitpath)
+            continue
+        sumdiff = len(gitroots.difference(my_roots)) + len(my_roots.difference(gitroots))
+        # If we only differ by a single root, consider us siblings
+        if sumdiff <= 2:
+            siblings.add(gitpath)
+            continue
 
     return siblings
 
 
-def find_best_obstrepo(mypath, obst_roots):
-    # We want to find a repo with best intersect len to total roots len ratio
+def find_best_obstrepo(mypath, obst_roots, minratio=0.2):
+    # We want to find a repo with best intersect len to total roots len ratio,
+    # but we'll ignore any repos where the ratio is too low, in order not to lump
+    # together repositories that have very weak common histories.
     myroots = get_repo_roots(mypath)
     if not myroots:
         return None
@@ -580,6 +591,8 @@ def find_best_obstrepo(mypath, obst_roots):
             continue
         icount = len(roots.intersection(myroots))
         ratio = icount / len(roots)
+        if ratio < minratio:
+            continue
         if ratio > bestratio:
             obstrepo = path
             bestratio = ratio
@@ -725,7 +738,7 @@ def is_obstrepo(fullpath, obstdir):
     return fullpath.find(obstdir) == 0
 
 
-def find_all_gitdirs(toplevel, ignore=None, normalize=False, exclude_objstore=True, flat=False):
+def find_all_gitdirs(toplevel, ignore=None, normalize=False, exclude_objstore=True):
     global _alt_repo_map
     if _alt_repo_map is None:
         _alt_repo_map = dict()
@@ -739,39 +752,44 @@ def find_all_gitdirs(toplevel, ignore=None, normalize=False, exclude_objstore=Tr
     logger.info('   search: finding all repos in %s', toplevel)
     logger.debug('Ignore list: %s', ' '.join(ignore))
     gitdirs = set()
-    tp = pathlib.Path(toplevel)
-    if flat:
-        globpatt = '*.git'
-    else:
-        globpatt = '**/*.git'
-    for subp in tp.glob(globpatt):
-        # Should we ignore this dir?
-        ignored = False
-        for ignoreglob in ignore:
-            if subp.match(ignoreglob):
-                ignored = True
-                break
-        if ignored:
+    for root, dirs, files in os.walk(toplevel, topdown=True):
+        if not len(dirs):
             continue
-        fullpath = subp.resolve().as_posix()
-        if not is_bare_git_repo(fullpath):
-            continue
-        if exclude_objstore and os.path.exists(os.path.join(fullpath, 'grokmirror.objstore')):
-            continue
-        if normalize:
-            fullpath = os.path.realpath(fullpath)
 
-        logger.debug('Found %s', fullpath)
-        if fullpath not in gitdirs:
-            gitdirs.add(fullpath)
-
-        if build_amap:
-            altrepo = get_altrepo(fullpath)
-            if not altrepo:
+        torm = set()
+        for name in dirs:
+            fullpath = os.path.join(root, name)
+            # Should we ignore this dir?
+            ignored = False
+            for ignoredir in ignore:
+                if fnmatch.fnmatch(fullpath, ignoredir):
+                    torm.add(name)
+                    ignored = True
+                    break
+            if ignored:
                 continue
-            if altrepo not in _alt_repo_map:
-                _alt_repo_map[altrepo] = set()
-            _alt_repo_map[altrepo].add(fullpath)
+            if not is_bare_git_repo(fullpath):
+                continue
+            if exclude_objstore and os.path.exists(os.path.join(fullpath, 'grokmirror.objstore')):
+                continue
+            if normalize:
+                fullpath = os.path.realpath(fullpath)
+
+            logger.debug('Found %s', os.path.join(root, name))
+            gitdirs.add(fullpath)
+            torm.add(name)
+
+            if build_amap:
+                altrepo = get_altrepo(fullpath)
+                if not altrepo:
+                    continue
+                if altrepo not in _alt_repo_map:
+                    _alt_repo_map[altrepo] = set()
+                _alt_repo_map[altrepo].add(fullpath)
+
+        for name in torm:
+            # don't recurse into the found *.git dirs
+            dirs.remove(name)
 
     return gitdirs
 
